@@ -6,7 +6,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2024	PgPool Global Development Group
+ * Copyright (c) 2003-2021	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -39,7 +39,7 @@
  * block". Each block is assigned a "cache block id", which is
  * starting with 0.
  */
-typedef char *POOL_CACHE_BLOCK;	/* pointer to cache block */
+typedef char *POOL_CACH_BLOCK;	/* pointer to cache block */
 typedef unsigned int POOL_CACHE_BLOCKID;	/* cache block id */
 typedef unsigned int POOL_CACHE_ITEMID; /* cache item id */
 
@@ -59,9 +59,6 @@ typedef struct
 
 #define POOL_BLOCK_USED	0x0001	/* is this block used? */
 
-/*
- * Cache block header (12 bytes)
- */
 typedef struct
 {
 	unsigned char flags;		/* flags. see above */
@@ -78,9 +75,6 @@ typedef struct
 #define POOL_ITEM_HAS_NEXT	0x0002	/* is this item has "next" item? */
 #define POOL_ITEM_DELETED	0x0004	/* is this item deleted? */
 
-/*
- * Cache item pointer (48 bytes)
- */
 typedef struct
 {
 	POOL_QUERY_HASH query_hash; /* md5 hashed query signature */
@@ -91,7 +85,7 @@ typedef struct
 
 /*
  * Each block holds several "cache item", which consists of variable
- * length of Data (header plus RowDescription packet and DataRow
+ * length of Data(header plus RowDescription packet and DataRow
  * packet).  Each cache item is assigned "cache item id", which
  * represents the cache item order in a block.
  */
@@ -104,13 +98,12 @@ typedef struct
 
 /*
  * "Cache Item header" structure is used to manage each cache item.
- *  (16 bytes)
  */
 typedef struct
 {
 	unsigned int total_length;	/* total length in bytes including myself */
 	time_t		timestamp;		/* cache creation time */
-	int64		expire;			/* cache expire	duration in seconds */
+	int			expire;			/* cache expire	*/
 }			POOL_CACHE_ITEM_HEADER;
 
 typedef struct
@@ -211,7 +204,7 @@ typedef struct
  *--------------------------------------------------------------------------------
  */
 
-/* Hash element (48 bytes) */
+/* Hash element */
 typedef struct POOL_HASH_ELEMENT
 {
 	struct POOL_HASH_ELEMENT *next; /* link to next entry */
@@ -221,7 +214,7 @@ typedef struct POOL_HASH_ELEMENT
 
 typedef uint32 POOL_HASH_KEY;
 
-/* Hash header element (16 bytes) */
+/* Hash header element */
 typedef struct
 {
 	POOL_HASH_KEY hashkey;		/* hash key */
@@ -236,6 +229,12 @@ typedef struct
 	POOL_HEADER_ELEMENT elements[1];	/* actual hash elements follows */
 }			POOL_HASH_HEADER;
 
+typedef enum
+{
+	POOL_MEMQ_SHARED_LOCK = 0,
+	POOL_MEMQ_EXCLUSIVE_LOCK,
+} POOL_MEMQ_LOCK_TYPE;
+
 extern int	pool_hash_init(int nelements);
 extern size_t pool_hash_size(int nelements);
 extern POOL_CACHEID * pool_hash_search(POOL_QUERY_HASH * key);
@@ -244,7 +243,7 @@ extern uint32 hash_any(unsigned char *k, int keylen);
 
 extern POOL_STATUS pool_fetch_from_memory_cache(POOL_CONNECTION * frontend,
 												POOL_CONNECTION_POOL * backend,
-												char *contents, bool use_fake_cache, bool *foundp);
+												char *contents, bool *foundp);
 
 extern int pool_fetch_cache(POOL_CONNECTION_POOL * backend, const char *query, char **buf, size_t *len);
 extern int pool_catalog_commit_cache(POOL_CONNECTION_POOL * backend, char *query, char *data, size_t datalen);
@@ -271,8 +270,7 @@ extern POOL_QUERY_CACHE_ARRAY * pool_create_query_cache_array(void);
 extern void pool_discard_query_cache_array(POOL_QUERY_CACHE_ARRAY * cache_array);
 
 extern POOL_TEMP_QUERY_CACHE * pool_create_temp_query_cache(char *query);
-extern void pool_handle_query_cache(POOL_CONNECTION_POOL * backend, char *query, Node *node, char state,
-	bool partial_fetch);
+extern void pool_handle_query_cache(POOL_CONNECTION_POOL * backend, char *query, Node *node, char state);
 
 extern int	pool_init_memqcache_stats(void);
 extern POOL_QUERY_CACHE_STATS * pool_get_memqcache_stats(void);
@@ -289,14 +287,10 @@ extern POOL_TEMP_QUERY_CACHE * pool_get_current_cache(void);
 extern void pool_discard_temp_query_cache(POOL_TEMP_QUERY_CACHE * temp_cache);
 extern void pool_discard_current_temp_query_cache(void);
 
-extern void pool_shmem_lock(void);
+extern void pool_shmem_lock(POOL_MEMQ_LOCK_TYPE type);
 extern void pool_shmem_unlock(void);
 extern bool pool_is_shmem_lock(void);
 
 extern void InvalidateQueryCache(int tableoid, int dboid);
 
-extern void pool_init_whole_cache_blocks(void);
-
-extern int delete_all_cache_on_memcached(void);
-
 #endif							/* POOL_MEMQCACHE_H */
