@@ -3,7 +3,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2026	PgPool Global Development Group
+ * Copyright (c) 2003-2021	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -46,8 +46,6 @@ static bool is_system_catalog(char *table_name);
 static bool temp_table_walker(Node *node, void *context);
 static bool unlogged_table_walker(Node *node, void *context);
 static bool view_walker(Node *node, void *context);
-static bool row_security_enabled(char *table_name);
-static bool row_security_enabled_walker(Node *node, void *context);
 static bool is_temp_table(char *table_name);
 static bool insertinto_or_locking_clause_walker(Node *node, void *context);
 static bool is_immutable_function(char *fname);
@@ -174,25 +172,6 @@ pool_has_view(Node *node)
 	return ctx.has_view;
 }
 
-/*
- * Return true if this SELECT has a row security enabled table.
- */
-bool
-pool_has_row_security(Node *node)
-{
-
-	SelectContext ctx;
-
-	if (!IsA(node, SelectStmt))
-		return false;
-
-	ctx.row_security = false;
-
-	raw_expression_tree_walker(node, row_security_enabled_walker, &ctx);
-
-	return ctx.row_security;
-}
-
 /*
  * Return true if this SELECT has INSERT INTO or FOR SHARE or FOR UPDATE.
  */
@@ -387,9 +366,9 @@ function_call_walker(Node *node, void *context)
 					Node	   *arg = linitial(fcall->args);
 
 					if (IsA(arg, A_Const) &&
-						((A_Const *) arg)->val.type == T_Integer)
+						IsA(&((A_Const *) arg)->val, Integer))
 					{
-						ctx->pg_terminate_backend_pid = ((A_Const *) arg)->val.val.ival;
+						ctx->pg_terminate_backend_pid = ((A_Const *) arg)->val.ival.ival;
 						ereport(DEBUG1,
 								(errmsg("pg_terminate_backend pid = %d", ctx->pg_terminate_backend_pid)));
 					}
@@ -406,7 +385,7 @@ function_call_walker(Node *node, void *context)
 				if (function_volatile_property(fname, FUNC_VOLATILE))
 				{
 					ctx->has_function_call = true;
-					return true;
+					return false;
 				}
 				return raw_expression_tree_walker(node, function_call_walker, context);
 			}
@@ -431,7 +410,7 @@ function_call_walker(Node *node, void *context)
 				 * found a writing function.
 				 */
 				ctx->has_function_call = true;
-				return true;
+				return false;
 			}
 
 			/*
@@ -444,7 +423,7 @@ function_call_walker(Node *node, void *context)
 				{
 					/* Found. */
 					ctx->has_function_call = true;
-					return true;
+					return false;
 				}
 			}
 		}
@@ -473,7 +452,7 @@ system_catalog_walker(Node *node, void *context)
 		if (is_system_catalog(rgv->relname))
 		{
 			ctx->has_system_catalog = true;
-			return true;
+			return false;
 		}
 	}
 	return raw_expression_tree_walker(node, system_catalog_walker, context);
@@ -500,7 +479,7 @@ temp_table_walker(Node *node, void *context)
 		if (is_temp_table(rgv->relname))
 		{
 			ctx->has_temp_table = true;
-			return true;
+			return false;
 		}
 	}
 	return raw_expression_tree_walker(node, temp_table_walker, context);
@@ -530,7 +509,7 @@ unlogged_table_walker(Node *node, void *context)
 		if (is_unlogged_table(relname))
 		{
 			ctx->has_unlogged_table = true;
-			return true;
+			return false;
 		}
 	}
 	return raw_expression_tree_walker(node, unlogged_table_walker, context);
@@ -560,42 +539,12 @@ view_walker(Node *node, void *context)
 		if (is_view(relname))
 		{
 			ctx->has_view = true;
-			return true;
+			return false;
 		}
 	}
 	return raw_expression_tree_walker(node, view_walker, context);
 }
 
-/*
- * Walker function to find a row security enabled table.
- */
-static bool
-row_security_enabled_walker(Node *node, void *context)
-{
-	SelectContext *ctx = (SelectContext *) context;
-	char	   *relname;
-
-	if (node == NULL)
-		return false;
-
-	if (IsA(node, RangeVar))
-	{
-		RangeVar   *rgv = (RangeVar *) node;
-
-		relname = make_table_name_from_rangevar(rgv);
-
-		ereport(DEBUG1,
-				(errmsg("row secuirty walker. checking relation \"%s\"", relname)));
-
-		if (row_security_enabled(relname))
-		{
-			ctx->row_security = true;
-			return true;
-		}
-	}
-	return raw_expression_tree_walker(node, row_security_enabled_walker, context);
-}
-
 /*
  * Determine whether table_name is a system catalog or not.
  */
@@ -610,11 +559,11 @@ is_system_catalog(char *table_name)
 /*
  * Query to know if the target table belongs pg_catalog schema.
  */
-#define ISBELONGTOPGCATALOGQUERY "SELECT count(*) FROM pg_catalog.pg_class AS c, pg_catalog.pg_namespace AS n WHERE c.relname = '%s' AND c.relnamespace = n.oid AND n.nspname = 'pg_catalog'"
+#define ISBELONGTOPGCATALOGQUERY "SELECT count(*) FROM pg_class AS c, pg_namespace AS n WHERE c.relname = '%s' AND c.relnamespace = n.oid AND n.nspname = 'pg_catalog'"
 
-#define ISBELONGTOPGCATALOGQUERY2 "SELECT count(*) FROM pg_catalog.pg_class AS c, pg_catalog.pg_namespace AS n WHERE c.oid = pgpool_regclass('\"%s\"') AND c.relnamespace = n.oid AND n.nspname = 'pg_catalog'"
+#define ISBELONGTOPGCATALOGQUERY2 "SELECT count(*) FROM pg_class AS c, pg_namespace AS n WHERE c.oid = pgpool_regclass('\"%s\"') AND c.relnamespace = n.oid AND n.nspname = 'pg_catalog'"
 
-#define ISBELONGTOPGCATALOGQUERY3 "SELECT count(*) FROM pg_catalog.pg_class AS c, pg_catalog.pg_namespace AS n WHERE c.oid = pg_catalog.to_regclass('\"%s\"') AND c.relnamespace = n.oid AND n.nspname = 'pg_catalog'"
+#define ISBELONGTOPGCATALOGQUERY3 "SELECT count(*) FROM pg_class AS c, pg_namespace AS n WHERE c.oid = pg_catalog.to_regclass('\"%s\"') AND c.relnamespace = n.oid AND n.nspname = 'pg_catalog'"
 
 	bool		result;
 	static POOL_RELCACHE * relcache;
@@ -701,7 +650,7 @@ is_temp_table(char *table_name)
  * regclass (or its variant) here, because temporary tables never have
  * schema qualified name.
  */
-#define ISTEMPQUERY83 "SELECT count(*) FROM pg_catalog.pg_class AS c, pg_catalog.pg_namespace AS n WHERE c.relname = '%s' AND c.relnamespace = n.oid AND n.nspname ~ '^pg_temp_'"
+#define ISTEMPQUERY83 "SELECT count(*) FROM pg_catalog.pg_class AS c, pg_namespace AS n WHERE c.relname = '%s' AND c.relnamespace = n.oid AND n.nspname ~ '^pg_temp_'"
 
 /*
  * Query to know if the target table is a temporary one.  This query
@@ -953,60 +902,6 @@ is_view(char *table_name)
 	return result;
 }
 
-/*
- * Returns true if table_name enables row security.
- */
-static bool
-row_security_enabled(char *table_name)
-{
-/*
- * Query to know if the target table enables row security. This is valid for
- * PostgreSQL 9.5 or later. Remember that to_regclass() is available in
- * PostgreSQL 9.4 or later and we can use to_regclass() unconditionally.
- */
-#define ISROWSECURITYQUERY "SELECT count(*) FROM pg_catalog.pg_class AS c WHERE c.oid = pg_catalog.to_regclass('%s') AND c.relrowsecurity"
-
-	static POOL_RELCACHE * relcache;
-	POOL_CONNECTION_POOL *backend;
-	bool		result;
-	char	   *query;
-
-	if (table_name == NULL)
-		return false;
-
-	backend = pool_get_session_context(false)->backend;
-
-	/*
-	 * Check backend version. PostgreSQL 9.5 or later have relrowsecurity
-	 * column.
-	 */
-	if (Pgversion(backend)->major < 95)
-		return false;
-
-	query = ISROWSECURITYQUERY;
-
-	if (!relcache)
-	{
-		relcache = pool_create_relcache(pool_config->relcache_size, query,
-										int_register_func, int_unregister_func,
-										false);
-		if (relcache == NULL)
-		{
-			ereport(WARNING,
-					(errmsg("unable to create relcache, while checking for row security")));
-			return false;
-		}
-
-	}
-
-	/*
-	 * Search relcache.
-	 */
-	result = pool_search_relcache(relcache, backend, table_name) == 0 ? false : true;
-	return result;
-}
-
-
 /*
  * Judge if we have pgpool_regclass or not.
  */
@@ -1016,7 +911,7 @@ pool_has_pgpool_regclass(void)
 /*
  * Query to know if pgpool_regclass exists.
  */
-#define HASPGPOOL_REGCLASSQUERY "SELECT count(*) from (SELECT pg_catalog.has_function_privilege('%s', 'pgpool_regclass(cstring)', 'execute') WHERE EXISTS(SELECT * FROM pg_catalog.pg_proc AS p WHERE p.proname = 'pgpool_regclass')) AS s"
+#define HASPGPOOL_REGCLASSQUERY "SELECT count(*) from (SELECT has_function_privilege('%s', 'pgpool_regclass(cstring)', 'execute') WHERE EXISTS(SELECT * FROM pg_catalog.pg_proc AS p WHERE p.proname = 'pgpool_regclass')) AS s"
 
 	bool		result;
 	static POOL_RELCACHE * relcache;
@@ -1304,7 +1199,7 @@ pool_table_name_to_oid(char *table_name)
  * Query to convert table name to oid
  */
 #define TABLE_TO_OID_QUERY "SELECT pgpool_regclass('%s')"
-#define TABLE_TO_OID_QUERY2 "SELECT oid FROM pg_catalog.pg_class WHERE relname = '%s'"
+#define TABLE_TO_OID_QUERY2 "SELECT oid FROM pg_class WHERE relname = '%s'"
 #define TABLE_TO_OID_QUERY3 "SELECT COALESCE(pg_catalog.to_regclass('%s')::oid, 0)"
 
 	int			oid = 0;
@@ -1419,7 +1314,7 @@ select_table_walker(Node *node, void *context)
 			num_oids = ctx->num_oids++;
 
 			ctx->table_oids[num_oids] = oid;
-			strlcpy(ctx->table_names[num_oids], table, NAMEDATALEN);
+			strlcpy(ctx->table_names[num_oids], table, POOL_NAMEDATALEN);
 
 			ereport(DEBUG1,
 					(errmsg("extracting table oids from SELECT statement"),
@@ -1482,11 +1377,11 @@ make_function_name_from_funccall(FuncCall *fcall)
 {
 	/*
 	 * Function name. Max size is calculated as follows: schema
-	 * name(NAMEDATALEN byte) + quotation marks for schema name(2 byte) +
-	 * period(1 byte) + table name (NAMEDATALEN byte) + quotation marks
+	 * name(POOL_NAMEDATALEN byte) + quotation marks for schema name(2 byte) +
+	 * period(1 byte) + table name (POOL_NAMEDATALEN byte) + quotation marks
 	 * for table name(2 byte) + NULL(1 byte)
 	 */
-	static char funcname[NAMEDATALEN * 2 + 1 + 2 * 2 + 1];
+	static char funcname[POOL_NAMEDATALEN * 2 + 1 + 2 * 2 + 1];
 	List 	   *names;
 
 	if(fcall == NULL)
@@ -1503,29 +1398,29 @@ make_function_name_from_funccall(FuncCall *fcall)
 	{
 		case 1:
 			strcat(funcname, "\"");
-			strncat(funcname, strVal(linitial(names)), NAMEDATALEN);
+			strncat(funcname, strVal(linitial(names)), POOL_NAMEDATALEN);
 			strcat(funcname, "\"");
 			break;
 		case 2:
 			strcat(funcname, "\"");
-			strncat(funcname, strVal(linitial(names)), NAMEDATALEN);
+			strncat(funcname, strVal(linitial(names)), POOL_NAMEDATALEN);
 			strcat(funcname, "\"");
 
 			strcat(funcname, ".");
 
 			strcat(funcname, "\"");
-			strncat(funcname, strVal(lsecond(names)), NAMEDATALEN);
+			strncat(funcname, strVal(lsecond(names)), POOL_NAMEDATALEN);
 			strcat(funcname, "\"");
 			break;
 		case 3:
 			strcat(funcname, "\"");
-			strncat(funcname, strVal(lsecond(names)), NAMEDATALEN);
+			strncat(funcname, strVal(lsecond(names)), POOL_NAMEDATALEN);
 			strcat(funcname, "\"");
 
 			strcat(funcname, ".");
 
 			strcat(funcname, "\"");
-			strncat(funcname, strVal(lthird(names)), NAMEDATALEN);
+			strncat(funcname, strVal(lthird(names)), POOL_NAMEDATALEN);
 			strcat(funcname, "\"");
 
 			break;
@@ -1551,11 +1446,11 @@ make_table_name_from_rangevar(RangeVar *rangevar)
 {
 	/*
 	 * Table name. Max size is calculated as follows: schema
-	 * name(NAMEDATALEN byte) + quotation marks for schema name(2 byte) +
-	 * period(1 byte) + table name (NAMEDATALEN byte) + quotation marks
+	 * name(POOL_NAMEDATALEN byte) + quotation marks for schema name(2 byte) +
+	 * period(1 byte) + table name (POOL_NAMEDATALEN byte) + quotation marks
 	 * for table name(2 byte) + NULL(1 byte)
 	 */
-	static char tablename[NAMEDATALEN * 2 + 1 + 2 * 2 + 1];
+	static char tablename[POOL_NAMEDATALEN * 2 + 1 + 2 * 2 + 1];
 
 	if (rangevar == NULL)
 	{
@@ -1577,7 +1472,7 @@ make_table_name_from_rangevar(RangeVar *rangevar)
 	if (rangevar->schemaname)
 	{
 		strcat(tablename, "\"");
-		strncat(tablename, rangevar->schemaname, NAMEDATALEN);
+		strncat(tablename, rangevar->schemaname, POOL_NAMEDATALEN);
 		strcat(tablename, "\"");
 		strcat(tablename, ".");
 	}
@@ -1591,7 +1486,7 @@ make_table_name_from_rangevar(RangeVar *rangevar)
 	}
 
 	strcat(tablename, "\"");
-	strncat(tablename, rangevar->relname, NAMEDATALEN);
+	strncat(tablename, rangevar->relname, POOL_NAMEDATALEN);
 	strcat(tablename, "\"");
 
 	ereport(DEBUG1,
@@ -1610,7 +1505,7 @@ bool function_has_return_type(char *fname, char *typename)
 /*
  * Query to count the number of records matching given function name and type name.
  */
-#define FUNCTION_RETURN_TYPE_MATCHEING_QUERY "SELECT count(*) FROM pg_catalog.pg_type AS t, pg_catalog.pg_proc AS p, pg_catalog.pg_namespace AS n WHERE p.proname = '%s' AND n.oid = p.pronamespace AND n.nspname %s '%s' AND p.prorettype = t.oid AND t.typname = '%s';"
+#define FUNCTION_RETURN_TYPE_MATCHEING_QUERY "SELECT count(*) FROM pg_type AS t, pg_catalog.pg_proc AS p, pg_catalog.pg_namespace AS n WHERE p.proname = '%s' AND n.oid = p.pronamespace AND n.nspname %s '%s' AND p.prorettype = t.oid AND t.typname = '%s';"
 	bool		result;
 	char		query[1024];
 	char	   *rawstring = NULL;
